/*****************************************************************************
* Copyright (c) 2002-2004 Rafael de Pelegrini Soares.
* 
* LEGAL NOTICE: Licenced EMSO users have the right to use this file, but
* these coded instructions, statements, and computer programs contain
* proprietary information belonging to Rafael de Pelegrini Soares,
* and are protected by longernational copyright law.
* They may not be disclosed to third parties without the prior written
* consent of the copyright owner.
*
******************************************************************************
*
* $Id$
*
* This file contains util routines for using when developing EMSO interfaces
* in Fortran.
* The usage of these functions is exemplified in the Fortran template
* files external_solver.f, external_object.f.
*
* DO NOT modify this file, just include it in your fortran project and
* call the routines.
*
* STRING ROUTINES:
* Routines enabling the user exchange strings with EMSO. These routines are
* necessary because C and Fortran strings cannot be shared without a special
* treatment.
*
* The available functions are:
* EMSO_STR_READ: reads a string.
* EMSO_STR_WRITE: writes a string.
* EMSO_STR_VEC_WRITE: writes a string into a specified element of a string vector.
* EMSO_STR_VEC_READ: reads a spedified element of a string vector.
*
* It follows below a code piece showing how to use the functions:
*
* @code
* ! Writing the string "CriticalPressure" into the 'MethodName' string
*        CALL EMSO_STR_WRITE(MethodName, "CriticalPressure")
*
* ! Reading the 'MethodName' string and storing into the Fortran string STR
*        character*256 STR
*        CALL EMSO_STR_READ(MethodName, STR)
*
* ! Writing the string "kPa" into the first position one of a string vector.
*        CALL EMSO_STR_VEC_WRITE(OutputUnits, 1, "kPa")
*
* ! Reading the string at position 5 and storing into STR.
*        CALL EMSO_STR_VEC_READ(InputUnits, 5, STR)
* @endcode
*
* MEMORY ROUTINES:
* Routines enabling dynamic memory allocation.
* EMSO_MALLOC: returns the integer(*) for the first element of a
*    block of memory for a given number of BYTES.
*    The block can be used to store integer or real variables.
*    Remember that the given length represents the number of BYTES
*    of the block;
* EMSO_FREE: deallocate a previous allocated block of memory.
*
*
*******************************************************************************/


#include <stdlib.h>
#include <string.h>


/** This routines enables fortran codes to set a strings
* in a vector of strings.
*
* It follows below a sample piece of code:
* @code
* CALL EMSO_STR_VEC_WRITE(STRING_VECTOR, 1, "Text")
* @endcode
*/
void
#ifdef _MSC_VER
__stdcall EMSO_STR_VEC_WRITE
#else
emso_str_vec_write_
#endif
(int *strings, int *position, const char *value, int len){
	char **string_vector = (char **) strings;
	if(*position>0){
		strncpy(string_vector[*position-1], value, len);
		string_vector[*position-1][len] = '\0';
	}
}

void
#ifdef _MSC_VER
__stdcall EMSO_STR_VEC_READ
#else
emso_str_vec_read_
#endif
(int *strings, int *position, char *value, int len){
	char **string_vector = (char **) strings;
	if(*position>0)
		strncpy(value, string_vector[*position-1], len);
}

void
#ifdef _MSC_VER
__stdcall EMSO_STR_WRITE
#else
emso_str_write_
#endif
(int *string, const char *value, int len){
	strncpy((char*)string, value, len);
	((char*)string)[len] = '\0';
}

void
#ifdef _MSC_VER
__stdcall EMSO_STR_READ
#else
emso_str_read_
#endif
(int *string, char *value, int len){
	strncpy(value, (char*)string, len);
}


int*
#ifdef _MSC_VER
__stdcall EMSO_MALLOC
#else
emso_malloc
#endif
(int *len){
	return (int*) malloc(*len);
}

void
#ifdef _MSC_VER
__stdcall EMSO_FREE
#else
emso_free
#endif
(int *mem){
	free((void*) *mem);
}


