/*
* This file is part of the EMSO project and cannot be
* used, copyed or modified without the permission of
* the author.
*
* Copyright (C) 2004-2005  Eduardo Lemos e Caio
*/
#include "external_object.h"

#include <cstdio>
#include <cstring>
#include <cmath>

#ifdef WIN32
# include <windows.h>
# define snprintf _snprintf
#endif

#ifdef _MSC_VER
#define FCALL __stdcall
#define raizes       RAIZES
#define Matriz_AB    MATRIZ_AB
#define lagrange     LAGRANGE
#else
#define FCALL
#define raizes	     raizes_
#define Matriz_AB    matriz_ab_
#define lagrange     lagrange_
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern void FCALL raizes(int_t *n, int_t *nt, real_t *al, real_t *be, real_t *Raiz);
extern void FCALL Matriz_AB(int_t *n, int_t *nt, real_t *Raiz, real_t *A, real_t *B);
extern void FCALL lagrange(int_t *nt,real_t *Raiz,real_t *z,real_t *lag);

#ifdef __cplusplus
}
#endif

/** Data structure the Object.
* This object has only some methods:
*  - method1: receives T and returns T^2
*  - method2: receives T, P and returns T*P
*  - method3: receives z(5) and return sum(z)
*/
class ObjectOCFEM {
public:
	/// The constructor.
	ObjectOCFEM(){};

	/// Creates this object instance.
	virtual void create(int_t *retval, char *msg){
		++numOfPackages;
		InternalPoints_ = 3;
		NodalPoints_ = 3;
		alfa_ = 0;
		beta_ = 0;
		strcpy(boundary_, "NONE");
	}

	/// Destroyes this object instance.
	virtual void destroy(int_t *retval, char *msg){
		--numOfPackages;
		delete raiz;
		delete A;
		delete B;
		delete l;
	}

	/// Set a parameter for the instance.
	virtual void setParameter(const char *parameterName,
		const int_t *valueType,
		const int_t *valueLength,
		const real_t *values,
		const char *valuesText[],
		int_t *retval, char *msg) 
		{			// incluir testes a prova de usurio
			if (strcmp("InternalPoints", parameterName)==0){
				InternalPoints_ = (int_t)*values;  // checar conversao
			}else if (strcmp("Boundary", parameterName)==0){
				strcpy(boundary_, valuesText[0]);
			}else if (strcmp("alfa", parameterName)==0){
				alfa_ = (real_t)*values; 
			}else if (strcmp("beta", parameterName)==0){
				beta_ = (real_t)*values;
			} 
			
			else {
				*retval = emso_error;
				printf(msg, EMSO_MESSAGE_LENGTH, "invalid parameter for OCFEM Object");
			}

			if(strcmp("NONE", boundary_)==0) 
				NodalPoints_ = InternalPoints_;
			if(strcmp("UPPER", boundary_)==0) 
				NodalPoints_ = InternalPoints_ + 1;
			if(strcmp("LOWER", boundary_)==0) 
				NodalPoints_ = InternalPoints_ + 1;
			if(strcmp("BOTH", boundary_)==0) 
				NodalPoints_ = InternalPoints_ + 2;
			
		// ver sobre a alocao dos vetores raiz, A e B

			raiz = new real_t [NodalPoints_];
			A = new real_t [NodalPoints_*NodalPoints_];
			B = new real_t [NodalPoints_*NodalPoints_];
			l = new real_t [NodalPoints_];
							
			raizes (&InternalPoints_, &NodalPoints_, &alfa_, &beta_, raiz);
			Matriz_AB (&InternalPoints_, &NodalPoints_, raiz, A, B);
			
			// aplicar fator de conversao na raiz
			// aplicar fator de comverso em A e em B
		}

	/// Check method.
	virtual void checkMethod(const char *methodName,
		int_t *methodID,
		int_t *numOfInputs,
		int_t *numOfOutputs,
		int_t *retval, char *msg){
			
			int_t method;
			
			*methodID = method = convertMethod(methodName);
			
			if(method == unknownMethod){
				*retval = emso_error;
				printf(msg, EMSO_MESSAGE_LENGTH, "method '%s' not found", methodName);
				return;
			}
			
			// the number of inputs	and outputs
			switch(method){
				case roots:
				case matrixA:
				case matrixB:
				case InternalPoints:
				case NodalPoints:
					*numOfInputs = 0;
					*numOfOutputs = 1;
					 break;
				case Lagrange:
					*numOfInputs = 1;
					*numOfOutputs = 1;
				break;
			}
		}
	
	/// Method details.
	virtual void methodDetails(const char *methodName,
		const int_t *methodID,

		const int_t *numOfInputs,
		int_t *inputLengths,
		int_t *inputTypes,
		char *inputUnits[],

		const int_t *numOfOutputs,
		int_t *outputLengths,
		int_t *outputTypes,
		char *outputUnits[],
		int_t *derivativeMatrix,

		int_t *retval, char *msg){
			
		int_t method;

		method = *methodID;
		if(!method)
			method = convertMethod(methodName);

		if(method == unknownMethod){
			*retval = emso_error;
			printf(msg, EMSO_MESSAGE_LENGTH, "method '%s' not found", methodName);
			return;
		}

		// the method units
		switch(method){
			case roots:
//				strcpy(outputUnits[0], "m");  // verificar as unidades entre EMSO e FORTRAN
				outputLengths[0] = NodalPoints_;
				break;
				// ver se ficar com ou sem unidades
			case matrixA:
//				strcpy(outputUnits[0], "1/m");
				outputLengths[0] = NodalPoints_*NodalPoints_;  // checar dimensao matriz
				break;
			case matrixB:
//				strcpy(outputUnits[0], "1/m^2");
				outputLengths[0] = NodalPoints_*NodalPoints_;  // checar dimensao matriz
				break;
			case Lagrange:
				inputLengths[0] = 1;								
				outputLengths[0] = NodalPoints_;								
				break;
			case InternalPoints:
			case NodalPoints:
				break;
		}
	}

	/// Calculation.
	virtual void calc(const char *methodName,
		const int_t *methodID,
		
		const int_t *numOfInputs,
		const int_t *inputLengths,
		const int_t *totalInputLength,
		const real_t *inputValues,

		const int_t *numOfOutputs,
		const int_t *outputLengths,
		const int_t *totalOutputLength,
		real_t *outputValues,

		const int_t *calculeDerivatives,
		real_t *outputDerivatives,

		int_t *retval, char *msg){
			
		int_t method;
		method = *methodID;
		if(!method)
			method = convertMethod(methodName);

		if(method == unknownMethod){
			*retval = emso_error;
			printf(msg, EMSO_MESSAGE_LENGTH, "method '%s' not found", methodName);
			return;
		}
		
		
		switch(method){
			case roots:
				memcpy(outputValues, raiz, NodalPoints_*sizeof(real_t));
				break;
				// ver a questao de A e B depender de outro mtodo(raiz) e se fica melhor juntos ou separados
			case matrixA:
				memcpy(outputValues, A, NodalPoints_*NodalPoints_*sizeof(real_t));
				break;
			case matrixB:
				memcpy(outputValues, B, NodalPoints_*NodalPoints_*sizeof(real_t));
				break;
			case Lagrange:
				lagrange(&NodalPoints_, raiz, (real_t *)inputValues , l);
				memcpy(outputValues, l, NodalPoints_*sizeof(real_t));
				break;
			case InternalPoints:
				outputValues[0] = InternalPoints_;
				break;
			case NodalPoints:
				outputValues[0] = NodalPoints_;
				break;
		}
	}

// virtual void destroy ()
// delete raiz;

private:
	/// The current number of created package instances.
	static int_t numOfPackages;
	
	/// The methods.
	enum Methods {
		roots,
		InternalPoints,
		matrixA,
		matrixB,
		Lagrange,
		NodalPoints,
		unknownMethod = 0xffff
	};
	// ver a questo dos valores defaulf que est dando problemas
	int_t InternalPoints_;
	int_t NodalPoints_;
	char boundary_[6];
	real_t alfa_, beta_, *raiz, *A, *B, *l;

	/** Converts a name to one of the enumerators #method_names
	* If the given name is not a valid method #unknownMethod is returned.
	*/
	int_t convertMethod(const char *name){
		if(strcmp("roots", name)==0) return roots;
		if(strcmp("matrixA", name)==0) return matrixA;
		if(strcmp("matrixB", name)==0) return matrixB;
		if(strcmp("Lagrange", name)==0) return Lagrange;
		if(strcmp("InternalPoints", name)==0) return InternalPoints;
		if(strcmp("NodalPoints", name)==0) return NodalPoints;
		
		return unknownMethod;
	}

	/** check if we have an error and print it to the given msg
	*
	*/
	void report_error(int_t *retval, char *msg, const char *e){
		*retval = emso_error;
		printf(msg, EMSO_MESSAGE_LENGTH, e);
	} 

};

// Initialization of the number of packages
int_t ObjectOCFEM::numOfPackages = 0;

/*******************************************************************************
*
* Herein after follows the C interface wrapping the C++ implementation,
* the user should NOT modify these functions.
*/

// Keep this function as is.
void eo_create_ (int_t *objectHandler,
				 int_t *retval, char *msg){
	// Create the objecty instance using the factory.
	ObjectOCFEM *obj = new ObjectOCFEM();

	// Call the correspounding c++ function.
	obj->create(retval, msg);

	// Return our object address.
	*objectHandler = (int_t)obj;
}

// Keep this function as is.
void eo_destroy_ (const int_t *objectHandler,
                  int_t *retval, char *msg){
	// Cast back the object.
	ObjectOCFEM *obj = (ObjectOCFEM*) *objectHandler;

	// Call the correspounding c++ function.
	obj->destroy(retval, msg);
	// Delete the object instance.
	delete obj;
}

// Keep this function as is.
void eo_set_parameter_ (const int_t *objectHandler,
						const char *parameterName,
						const int_t *valueType,
						const int_t *valueLength,
						const real_t *values,
						const char *valuesText[],
						int_t *retval, char *msg){
	// Cast back the object.
	ObjectOCFEM *obj = (ObjectOCFEM*) *objectHandler;

	// Call the correspounding c++ function.
	obj->setParameter(parameterName, valueType, valueLength,
		values, valuesText, retval, msg);
}

// Keep this function as is.
void eo_check_method_(const int_t *objectHandler,
					  const char *methodName,
					  int_t *methodID,
					  int_t *numOfInputs,
					  int_t *numOfOutputs,
					  int_t *retval, char *msg
){
	// Cast back the object.
	ObjectOCFEM *obj = (ObjectOCFEM *)*objectHandler;

	// Call the correspounding c++ function.
	obj->checkMethod(methodName, methodID,
		numOfInputs, numOfOutputs, retval, msg);
}

// Keep this function as is.
void eo_method_details_(const int_t *objectHandler,
						const char *methodName,
						const int_t *methodID,

						const int_t *numOfInputs,
						int_t *inputLengths,
						int_t *inputTypes,
						char *inputUnits[],

						const int_t *numOfOutputs,
						int_t *outputLengths,
						int_t *outputTypes,
						char *outputUnits[],
						int_t *derivativeMatrix,

						int_t *retval, char *msg
){
	// Cast back the object.
	ObjectOCFEM *obj = (ObjectOCFEM *)*objectHandler;

	// Call the correspounding c++ function.
	obj->methodDetails(methodName, methodID,
		numOfInputs, inputLengths, inputTypes, inputUnits,
		numOfOutputs, outputLengths, outputTypes, outputUnits,
		derivativeMatrix, retval, msg);
}

// Keep this function as is.
void eo_calc_(const int_t *objectHandler,
			  const char *methodName,
			  const int_t *methodID,

			  const int_t *numOfInputs,
			  const int_t *inputLengths,
			  const int_t *totalInputLength,
			  const real_t *inputValues,

			  const int_t *numOfOutputs,
			  const int_t *outputLengths,
			  const int_t *totalOutputLength,
			  real_t *outputValues,

			  const int_t *calculateDerivatives,
			  real_t *outputDerivatives,

			  int_t *retval, char *msg
){
    // Cast back the object.
	ObjectOCFEM *obj = (ObjectOCFEM *)*objectHandler;

	// Call the correspounding c++ function.
	obj->calc(methodName, methodID,
		numOfInputs, inputLengths, totalInputLength, inputValues,
		numOfOutputs, outputLengths, totalOutputLength, outputValues,
		calculateDerivatives, outputDerivatives, retval, msg);
}
